/**
 * Browser.java is a part of the SOFA project.
 * This file was created by pepan on 14.4.2003.
 */
package SOFA.SOFAnode.InOut.Impl;

import SOFA.SOFAnode.TR.ComponentInfo;
import SOFA.Util.VMProperties;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;

/**
 * Browses TR and enables installing components from other TRs.
 * @author  Petr Panuska
 */
public class Browser extends JFrame implements ActionListener {

  /**
   * A list of known SOFA nodes.
   */
  private Properties TRList;

  /**
   * The text appearing in the message area when starting this application.
   */
  private String messageText;

  /**
   * Used in JTrees.
   */
  private DefaultTreeModel model;

  /**
   * Used by a user for exploring known SOFA nodes and components stored in there and for
   * selecting these components before their distribution.
   */
  private JTree leftTR, rightTR;

  /**
   * Used by a user for pulling selected components to selected SOFA nodes.
   */
  private JButton obtainButton;

  /**
   * Used by a user for pushing selected components to selected SOFA nodes.
   */
  private JButton distributeButton;

  /**
   * Used by a user for pulling the description of selected components to selected SOFA nodes.
   */
  private JButton obtainOfferButton;

  /**
   * Used by a user for pushing the description of selected components to selected SOFA nodes.
   */
  private JButton distributeOfferButton;

  /**
   * Used by a user to force refreshing the content of JTrees.
   */
  private JButton refreshButton;

  /**
   * Shows messages (mainly errors) which occured during the running of the browser.
   */
  private static JTextArea messages;

  /**
   * Creates and initializes the browser.
   * @param confPath a path to the config file listing all known SOFA nodes.
   */
  public Browser (String confPath) {
    TRList = new Properties();
    messageText = "";
    try {
      TRList.load(new FileInputStream(confPath));
    } catch (IOException e) {
      messageText = "Configuration file not found (" + confPath + ")!\n";
      System.out.print(messageText);
    }

    obtainButton = new JButton("<<<");
    obtainButton.setToolTipText("Obtain selected components");
    obtainButton.addActionListener(this);
    distributeButton = new JButton(">>>");
    distributeButton.setToolTipText("Distribute selected components");
    distributeButton.addActionListener(this);

    obtainOfferButton = new JButton("<");
    obtainOfferButton.setToolTipText("Obtain description of selected components");
    obtainOfferButton.addActionListener(this);
    distributeOfferButton = new JButton(">");
    distributeOfferButton.setToolTipText("Distribute description of selected components");
    distributeOfferButton.addActionListener(this);

    refreshButton = new JButton("<>");
    refreshButton.setToolTipText("Refresh nodes");
    refreshButton.addActionListener(this);

    messages = new JTextArea(messageText);

    model = getTreeStructure(TRList);
    leftTR = new JTree(model);
    rightTR = new JTree(model);

    JSplitPane componentPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true,
      new JScrollPane(leftTR), new JScrollPane(rightTR));
    componentPanel.setResizeWeight(0.5);

    GridBagLayout gridBag = new GridBagLayout();
    JPanel panel = new JPanel(gridBag);

    GridBagConstraints c = new GridBagConstraints();

    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;
    c.gridwidth = GridBagConstraints.REMAINDER;
    gridBag.setConstraints(componentPanel, c);
    panel.add(componentPanel);

    c.fill = GridBagConstraints.NONE;
    c.gridwidth = 1;
    c.weighty = 0.0;
    c.anchor = GridBagConstraints.CENTER;
    gridBag.setConstraints(distributeOfferButton, c);
    panel.add(distributeOfferButton);

    gridBag.setConstraints(distributeButton, c);
    panel.add(distributeButton);

    gridBag.setConstraints(refreshButton, c);
    panel.add(refreshButton);

    gridBag.setConstraints(obtainButton, c);
    panel.add(obtainButton);

    c.gridwidth = GridBagConstraints.REMAINDER;
    gridBag.setConstraints(obtainOfferButton, c);
    panel.add(obtainOfferButton);

    JScrollPane messagePanel = new JScrollPane(messages);

    getContentPane().add(new JSplitPane(JSplitPane.VERTICAL_SPLIT, true, panel, messagePanel));
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    String sofaNodeName = System.getProperty(VMProperties.NODE_NAME);
    setTitle("SOFA Net Manager" + ((sofaNodeName == null) ? "" : " on " + sofaNodeName));
    pack();
    componentPanel.setDividerLocation(0.5);
    setVisible(true);

  }

  /**
   * Creates and returns the tree model used in both JTrees.
   * @param TRList the list of all known SOFA nodes. Each key is a unique name of
   * the entry; each value is a connector in a string form pointing from the client to
   * the InOut part of the SOFA node.
   * @return the tree model for both JTrees (all known SOFA nodes and all components stored
   * on them).
   */
  private DefaultTreeModel getTreeStructure (Properties TRList) {
    DefaultMutableTreeNode root = new DefaultMutableTreeNode("Known SOFA nodes");

    Enumeration enum = TRList.keys();
    while (enum.hasMoreElements()) {   // for all known sofa nodes
      String name = (String) enum.nextElement();
      String ref = (String) TRList.get(name);
      SOFANode node = new SOFANode(name, ref, messages);

      DefaultMutableTreeNode tNode = node.connect(root);
      if (node.isConnected())
        node.list(tNode);
    }

    return new DefaultTreeModel(root);
  }

  /**
   * Runs the browser. First it checks whether there is no more than one command
   * line parameter, otherwise it ends immediately. If no argumet is handed on to
   * the browser, it tries the default config file first ('tr.cfg' in the working directory).
   * @param argv one argument which is a path to the config file. If no argument is
   * used, the default one is tried.
   */
  public static void main (String[] argv) {
    if (argv.length > 1) {
      System.out.println(
        "Browser [config file name]\n" +
        "   - a config file of the name 'tr.cfg' in the working directory is used as default.");
      System.exit(-1);
    }
    String confPath = (argv.length == 0) ? "tr.cfg" : argv[0];

    new Browser(confPath);
  }

  /**
   * Receives actions performed by buttons.
   * @param e the emitted event. It also point to the button which has emitted the event.
   */
  public void actionPerformed (ActionEvent e) {
    if (e.getSource() == distributeButton || e.getSource() == obtainButton ||
      e.getSource() == distributeOfferButton || e.getSource() == obtainOfferButton) {
      JTree sourceTR, destTR;
      if (e.getSource() == distributeButton || e.getSource() == distributeOfferButton) {
        sourceTR = leftTR;
        destTR = rightTR;
      } else {
        sourceTR = rightTR;
        destTR = leftTR;
      }

      HashMap from = new HashMap();
      TreePath[] selected = sourceTR.getSelectionPaths();
      if (selected != null) {
        for (int i = 0; i < selected.length; i++) {
          TreePath path = selected[i];
          DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) path.getLastPathComponent();
          Object o = treeNode.getUserObject();
          if (o instanceof SOFANode) {
            SOFANode node = (SOFANode) o;
            HashSet components = new HashSet();
            ComponentInfo[] infos = node.getComponents();
            for (int j = 0; j < infos.length; j++) {
              ComponentInfo info = infos[j];
              components.add(info);
            }
            from.put(node, components);
          }
          if (o instanceof ComponentInfo) {
            ComponentInfo info = (ComponentInfo) o;
            DefaultMutableTreeNode treeNode2 = (DefaultMutableTreeNode) path.getParentPath().getLastPathComponent();
            SOFANode node = (SOFANode) treeNode2.getUserObject();
            HashSet components = (HashSet) from.get(node);
            if (components == null) {
              components = new HashSet();
              from.put(node, components);
            }
            components.add(info); // adds the object only if not already present.
          }
        }
      }
      HashSet to = new HashSet();
      selected = destTR.getSelectionPaths();
      if (selected != null) {
        for (int i = 0; i < selected.length; i++) {
          TreePath path = selected[i];
          DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) path.getLastPathComponent();
          Object o = treeNode.getUserObject();
          if (o instanceof SOFANode) {
            SOFANode node = (SOFANode) o;
            to.add(node);
          }
        }
      }
      if (e.getSource() == distributeButton)
        new DistributeDialog(from, to, true, false);
      else if (e.getSource() == distributeOfferButton)
        new DistributeDialog(from, to, true, true);
      else if (e.getSource() == obtainButton)
        new DistributeDialog(from, to, false, false);
      else if (e.getSource() == obtainOfferButton)
        new DistributeDialog(from, to, false, true);
      else
        assert(true);
    }
    if (e.getSource() == refreshButton) {
      DefaultMutableTreeNode root = (DefaultMutableTreeNode) model.getRoot();
      int size = root.getChildCount();
      for (int i = 0; i < size; i++) {
        DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) root.getChildAt(i);
        SOFANode sofaNode = (SOFANode) treeNode.getUserObject();
        if (!sofaNode.isConnected()) {
          sofaNode.reconnect(treeNode);
          model.nodeChanged(treeNode);
        }
        if (sofaNode.isConnected()) {
          sofaNode.list(treeNode);
          model.nodeStructureChanged(treeNode);
        }
      }
    }

  }

}
