/**
 * Main.java is a part of the SOFA project.
 * This file was created by pepan on 2.4.2003.
 */
package SOFA.SOFAnode.InOut.Impl;

import SOFA.Connector.ConnectorException;
import SOFA.Connector.Reference;
import SOFA.SOFAnode.InOut.Connector.InOut2ClientConnector;
import SOFA.SOFAnode.InOut.Connector.InOut2InOutConnector;
import SOFA.Util.VMProperties;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.RMISecurityManager;

/**
 * Runs the InOut part of a SOFA node. Allows two ways to run in - either if the <code>main</code>
 * method is used, then an RMI connector to TR is created, or the <code>runInOut</code> method is used,
 * then the local reference to TR can be provided.
 * @author Petr Panuska
 */
public class Main {

  /**
   * Runs InOut part of a SOFA node and connects it to its TR through an RMI connector.
   * @param argv no command parameters are to be expected. However, system properties
   * {@link VMProperties#RMI_HOST}, {@link VMProperties#RMI_PORT} and {@link VMProperties#TR_NAME}
   * according to the running TR may be specified. If not, the default values ('localhost', '1099' and
   * 'localTR') are used instead.
   */
  public static void main (String[] argv) {
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new RMISecurityManager());
    }
    String rmiHost = System.getProperty(VMProperties.RMI_HOST, "localhost");
    String rmiPort = System.getProperty(VMProperties.RMI_PORT, "1099");
    String trName = System.getProperty(VMProperties.TR_NAME, "localTR");
    runInOut(null, "SOFA.Connector.ECG.SGenerator.Connectors.DeplDockConnector-RMI;//"
      + rmiHost + ":" + rmiPort + "/SOFA/SOFAnode/TR/Connector/TR2InOutRMISkel/" + trName + "-");
  }

  /**
   * Runs InOut server. Just one parameter is set, the second one must be <code>null</code>.
   * @param trRef local reference to TR. If used, this InOut server is run in the same address space as
   * this TR - the connector uses a local connection.
   * @param referenceString a string form of a connector - remote reference to TR. If used,
   * this InOut server is run in a different address space than this TR - the connector uses
   * an RMI connection.
   */
  public static void runInOut (Reference trRef, String referenceString) {
    try {
      InOutImpl inOut = null;
      if (trRef == null) {
        String sofaNodeName = System.getProperty(VMProperties.NODE_NAME);

        if (sofaNodeName == null) {
          try {
            sofaNodeName = InetAddress.getLocalHost().getHostName();
          } catch (UnknownHostException e) {
            System.err.println("Can't get the host name.\nSet the name to the property '" + VMProperties.NODE_NAME + "' when TR is being started.");
            System.exit(1);
          }
        }

        System.setProperty(VMProperties.NODE_NAME, sofaNodeName);

        inOut = new InOutImpl(Reference.fromString(referenceString));
      } else
        inOut = new InOutImpl(trRef);

      System.out.print("Registering InOut2Client....");
      Reference refInOut = InOut2ClientConnector.createSrv(inOut).getSOFAReference();
      System.out.println("OK");
      System.out.println(refInOut);

      System.out.print("Registering InOut2InOut....");
      refInOut = InOut2InOutConnector.createSrv(inOut).getSOFAReference();
      System.out.println("OK");
      System.out.println(refInOut);

    } catch (ConnectorException e) {
      System.out.println("Exception " + e.getMessage());
      e.printStackTrace();
    }
/*
// todo following code is here for testing purposes only
    try {

      FileInputStream fis = new FileInputStream("c:\\ww\\t.tmp");
      byte[] buffer = new byte[fis.available()];
      fis.read(buffer);
      fis.close();
      ByteArrayInputStream bis = new ByteArrayInputStream(buffer);

      Reference refTR = new Reference();
      refTR._read(bis);
      bis.close();

      InOut2InOut inOut = new InOutImpl(refTR);
//      inOut.distributeComponent("::CUNI::SOFA::demos::logdemo::Logger[0.0.2]", null);
      ComponentInfo[] infos = {
        new ComponentInfoImpl("::CUNI::SOFA::demos::cplayer::ReadingUnit", "0.0.1"),
        new ComponentInfoImpl("::CUNI::SOFA::demos::logdemo::Tester", "0.0.1"),
        new ComponentInfoImpl("::CUNI::SOFA::demos::protodemo::LoggingWorker", "0.0.1"),
      };
      Bundle bundle = inOut.pullBundle(infos);
      inOut.pushBundle(bundle);

    } catch (Exception e) {
      e.printStackTrace();
      System.exit(-1);
    }
*/
  }
}
