/* $Id: TIRAccessMethods.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Access;


import SOFA.SOFAnode.Made.TIR.*;

import java.util.ArrayList;
import java.util.StringTokenizer;

import java.rmi.RemoteException;

/** Some methods for simple access to TIR.
  *
  * @author Petr Hnetynka
  */
public class TIRAccessMethods {

  /** Returns reference to TIR. This method use java properties sofa.rmiport and sofa.rmihost.
    * 
    * @return reference to TIR
    * @throws java.net.MalformedURLException bad url
    * @throws java.rmi.NotBoundException     repository does not exists
    * @throws java.rmi.RemoteException       rmote exception
    */
  public static Repository getRepository() throws java.net.MalformedURLException, java.rmi.NotBoundException, java.rmi.RemoteException {
    String rmiport = System.getProperty("sofa.rmiport","1099");
    String rmihost = System.getProperty("sofa.rmihost","localhost"); 
    return (Repository) java.rmi.Naming.lookup("//"+rmihost+":"+rmiport+"/Repository");
  }
  
  /** Search repository object.
    * @param rep     repository
    * @param lang    defining language of object (currently only <tt>cdl</tt> is possible)
    * @param name    full name without version (e.g. ::cz::cuni::sofa::demo)
    * @param version version of object
    * @exception RemoteException    remote exception
    * @exception TIRExceptLock      TIR is currently locked
    */
  public static Contained lookupContained(Repository rep, String lang, String name, String version) throws RemoteException, TIRExceptLock {
    ArrayList nm = new ArrayList();
    StringTokenizer st = new StringTokenizer(name,"::");
    while (st.hasMoreTokens()) {
      nm.add(st.nextToken());
    }

    Contained[] cnt;
    Container cont;
    Contained next = null;
    next = rep.lookup(lang,"");
    if (next == null) {
      return null;
    }
    cont = (Container) next;
    int i;
    for (i=0; i<nm.size()-1; i++) {
      cnt = cont.lookup_name((String) nm.get(i));
      if (cnt == null || cnt.length == 0) {
        return null;
      }  
      if (cnt.length == 1) {
        next = cnt[0];
      } else {
        next = cont.lookup((String) nm.get(i), version);
        if (next == null) {
          return null;
        }
      }
      if (!(next instanceof Container)) {
        return null;
      }
      cont = (Container) next;
    }
    return cont.lookup((String) nm.get(i), version);
  }
  
  /** Search repository object defined in CDL.
    * @param rep     repository
    * @param name    full name without version (e.g. ::cz::cuni::sofa::demo)
    * @param version version of object
    * @exception RemoteException    remote exception
    * @exception TIRExceptLock      TIR is currently locked
    */
  public static Contained lookupCDLContained(Repository rep, String name, String version) throws RemoteException, TIRExceptLock {
    return lookupContained(rep, "cdl", name, version);
  }
  
  /** Search repository object defined in CDL.
    * @param rep      repository
    * @param fullName full name with version (e.g. ::cz::cuni::sofa::demo?myhost!1)
    * @exception RemoteException    remote exception
    * @exception TIRExceptLock      TIR is currently locked
    */
  public static Contained lookupCDLContained(Repository rep, String fullName) throws RemoteException, TIRExceptLock {
    int n = fullName.indexOf('?');
    if (n != 0) {
      return lookupContained(rep, "cdl", fullName.substring(0,n), fullName.substring(n+1,fullName.length()));
    } else
      return null;
  }

  /** Return all operations and attributes (also inherited) of interface.
    * @param rep      repository
    * @param fullName full name with version of interface (e.g. ::cz::cuni::sofa::demo?myhost!1)
    * @exception RemoteException    remote exception
    * @exception TIRExceptLock      TIR is currently locked
    */
  public static Contained[] getOperAttrOfInterface(Repository rep, String fullName) throws RemoteException, TIRExceptLock {
    Contained cnt = lookupCDLContained(rep, fullName);
    if (cnt!=null && cnt.get_def_kind().value() == DefinitionKind.dk_Interface)
      return getOperAttrOfInterface((InterfaceDef) cnt);
    else
      return null;
  }
  
  /** Return all operations and attributes (also inherited) of interface.
    * @param iface      interface
    * @exception RemoteException    remote exception
    * @exception TIRExceptLock      TIR is currently locked
    */
  public static Contained[] getOperAttrOfInterface(InterfaceDef iface) throws RemoteException, TIRExceptLock {
    ArrayList all = new ArrayList();
    ArrayList scanned = new ArrayList();
    Contained[] cnt = iface.contents(null);
    for (int i=0; i<cnt.length; i++) {
      int dk = cnt[i].get_def_kind().value();
      if ( dk == DefinitionKind.dk_Attribute || dk == DefinitionKind.dk_Operation) {
        all.add(cnt[i]);
      }
    }
    scanned.add(iface);
    
    scanBase(all, scanned, iface);

    Contained[] ret = new Contained [all.size()];
    for (int i=0; i<ret.length; i++) {
      ret[i] = (Contained) all.get(i);
    }
    return ret;
  }

  private static void scanBase(ArrayList all, ArrayList scanned, InterfaceDef iface) throws RemoteException, TIRExceptLock {
    InterfaceDef[] base = iface.base_interfaces();
    for (int i=0; i< base.length; i++) {
      if (! isInScanned(scanned, base[i])) {
        Contained[] cnt = base[i].contents(null);
        for (int j=0; j<cnt.length; j++) {
          int dk = cnt[j].get_def_kind().value();
          if ( dk == DefinitionKind.dk_Attribute || dk == DefinitionKind.dk_Operation) {
            all.add(cnt[j]);
          }
        }
        scanned.add(base[i]);
        scanBase(all, scanned, base[i]);
      }
    }
  }

  private static boolean isInScanned(ArrayList scanned, InterfaceDef iface)  throws RemoteException {
    Identification id = iface.get_identification();
    for (int i=0; i<scanned.size(); i++) {
      if (((InterfaceDef) scanned.get(i)).get_identification().is_equal(id))
        return true;
    }
    return false;
  }

  /** Find interface of provision. It goes through typedefs and arrays.
    * 
    * @param prov   scanned provision
    * @return       provided interface
    * @throws RemoteException remote exception
    * @throws TIRExceptLock   TIR is currently locked
    */
  public static InterfaceDef getIfaceOfProvides(ProvideDef prov) throws RemoteException, TIRExceptLock {
    CDLType type = prov.provide();
    while (!(type instanceof InterfaceDef)) {
      int kind = type.get_def_kind().value();
      switch (kind) {
        case DefinitionKind.dk_Typedef:
          type = ((TypedefDef) type).original_type();
          break;
        case DefinitionKind.dk_Array:
          type = ((ArrayDef) type).element_type();
          break;
        case DefinitionKind.dk_Sequence:
          type = ((SequenceDef) type).element_type();
          break;
      }
    }
    return (InterfaceDef) type;
  }

  /** Scan provision for arrays/sequences
    * 
    * @param prov scanned provision
    * @return   true if provision contains arrays/sequences
    * @throws RemoteException remote exception
    * @throws TIRExceptLock   TIR is currently locked
    */
  public static boolean containProvArrays(ProvideDef prov) throws RemoteException, TIRExceptLock {
    CDLType type = prov.provide();
    while (!(type instanceof InterfaceDef)) {
      int kind = type.get_def_kind().value();
      switch (kind) {
        case DefinitionKind.dk_Typedef:
          type = ((TypedefDef) type).original_type();
          break;
        case DefinitionKind.dk_Array:
          return true;
        case DefinitionKind.dk_Sequence:
          return true;
      }
    }
    return false; 
  }


  /** Find interface of reqguirement. It goes through typedefs and arrays.
    * 
    * @param prov   scanned requirement
    * @return       required interface
    * @throws RemoteException remote exception
    * @throws TIRExceptLock   TIR is currently locked
    */
  public static InterfaceDef getIfaceOfRequires(RequireDef prov) throws RemoteException, TIRExceptLock {
    CDLType type = prov.require();
    while (!(type instanceof InterfaceDef)) {
      int kind = type.get_def_kind().value();
      switch (kind) {
        case DefinitionKind.dk_Typedef:
          type = ((TypedefDef) type).original_type();
          break;
        case DefinitionKind.dk_Array:
          type = ((ArrayDef) type).element_type();
          break;
        case DefinitionKind.dk_Sequence:
          type = ((SequenceDef) type).element_type();
          break;
      }
    }
    return (InterfaceDef) type;
  }

  /** Scan requirement for arrays/sequences
    * 
    * @param prov scanned requirement
    * @return   true if requirement contains arrays/sequences
    * @throws RemoteException remote exception
    * @throws TIRExceptLock   TIR is currently locked
    */
  public static boolean containReqArrays(RequireDef prov) throws RemoteException, TIRExceptLock {
    CDLType type = prov.require();
    while (!(type instanceof InterfaceDef)) {
      int kind = type.get_def_kind().value();
      switch (kind) {
        case DefinitionKind.dk_Typedef:
          type = ((TypedefDef) type).original_type();
          break;
        case DefinitionKind.dk_Array:
          return true;
        case DefinitionKind.dk_Sequence:
          return true;
      }
    }
    return false; 
  }

/*
  public static void main(String[] argv) {
    try {
      System.setSecurityManager(new java.rmi.RMISecurityManager());
    
      String rmiport = System.getProperty("tir.rmiport","1099");
      String rmihost = System.getProperty("tir.rmihost","localhost");
      Repository obj = (Repository) java.rmi.Naming.lookup("//"+rmihost+":"+rmiport+"/Repository");
      Contained[] cnt = getOperAttrOfInterface(obj, argv[0]);
      for (int i=0; i<cnt.length; i++) {
        System.out.println(cnt[i].get_identification().absolute_name().name());
      }
    } catch (Exception e) { 
      System.out.println("Exception: " + e.getMessage()); 
      e.printStackTrace(); 
    }
  }*/
}
