/* $Id: SpecFile.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Run;
import java.io.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

/** Representation of the SOFA specfile.
  * 
  * @author Petr Hnetynka
  */
public class SpecFile {
  
  /** Root element of the specfile */
  protected Element element;
  /** Implementation version */
  protected String releaseVersion;
  /** Full application name */
  protected String fullApplName;
  /** Full architecture name */
  protected String fullArchName;
  
  /** Constructor
    * 
    * @param elem  root element 
    */
  public SpecFile(Element elem) {
    element = elem;
    element.normalize();
    initialize();
    //debugPrint(element, "");
  }

  /** Initializing */
  private void initialize() {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("version")==0)) {
        releaseVersion = n.getFirstChild().getNodeValue();
      } else if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("architecture_ref")==0)) {
        String archName = n.getFirstChild().getNodeValue();
        fullArchName = archName;
        fullApplName = archName.substring(0, archName.indexOf('?'));
      }
    }
    fullApplName = fullApplName+"["+releaseVersion+"]";
  }

  /** Creates Specfile from deployment descriptor (!! it destroys dd).
    * 
    * @param dd   root element of depldescr
    * @return     new specfile
    */
  public static SpecFile getSpecFileFromDeplDescriptorXMLDoc(Element dd) {
    removeDDSpecificElements(dd);
    return new SpecFile(dd);
  }

  public static SpecFile getSpecFileFromFile(String fname) throws java.io.IOException, ParserConfigurationException, SAXException {
    java.io.File file = new java.io.File(fname);
    if (!file.exists()) {
      throw new java.io.IOException("File \""+fname+"\" doesn't exist.");
    }

    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = factory.newDocumentBuilder();
    Document document = builder.parse(file);
    Element element = document.getDocumentElement();
    element.normalize();
    return new SpecFile(element);
  }

  private static void removeDDSpecificElements(Node node) {
    NodeList nl = node.getChildNodes();
    java.util.ArrayList arr = new java.util.ArrayList();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && ((n.getNodeName().compareTo("cdl_entities")==0) || (n.getNodeName().compareTo("implementation")==0)) ) {
        arr.add(n);
      //  node.removeChild(n);
      } else if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("sofa_component")==0) ) {
        removeDDSpecificElements(n);
      } 
      if (n.getNodeType() == Node.TEXT_NODE) {
        arr.add(n);
        //node.removeChild(n);
      }
    }
    for (int i=0; i<arr.size(); i++) {
      node.removeChild((Node)arr.get(i));
    }
    node.normalize();
  }

  /** Writes specfile to file.
    * 
    * @param name name of the file
    * @throws IOException io exception
    */
  public void toFile(String name) throws IOException, TransformerConfigurationException, TransformerException {
    TransformerFactory tFactory = TransformerFactory.newInstance();
    Transformer transformer = tFactory.newTransformer();
    transformer.setOutputProperty("indent", "yes");
    transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "5");
    DOMSource source = new DOMSource(element);
    FileOutputStream out = new FileOutputStream(name);
    StreamResult result = new StreamResult(out);
    transformer.transform(source, result);
    out.close();
  }

  public void debugPrint(Node nd, String indent) {
    System.out.println(indent+nd.getNodeName()+" "+nodeType(nd));
    NodeList nl = nd.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      debugPrint(n, indent+"  ");
    } 
  }

  private String nodeType(Node nd) {
    switch (nd.getNodeType()) {
    case Node.ELEMENT_NODE: return "ELEMENT_NODE";
    case Node.TEXT_NODE: return "TEXT_NODE";
    default: return "UNKNOWN";
    }
  }

  /** Returns root element of the document.
    * 
    * @return root element
    */
  public Element getRootElement() {
    return element;
  }

  /** Returns full name of the architecture.
    * 
    * @return full name of the architecture
    */
  public String getFullArchName() {
    return fullArchName;
  }

  /** Returns implementation version.
    * 
    * @return implementation version
    */
  public String getImplVersion() {
    return releaseVersion;
  }

  public static String getFullApplName(Element element) {
    String releaseVersion = null;
    String fullApplName = null;
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("version")==0)) {
        releaseVersion = n.getFirstChild().getNodeValue();
      } else if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("architecture_ref")==0)) {
        String archName = n.getFirstChild().getNodeValue();
        fullApplName = archName.substring(0, archName.indexOf('?'));
      }
    }
    return fullApplName+"["+releaseVersion+"]";
  }

  public static String getFullArchName(Element element) {
    NodeList nl = element.getChildNodes();
    for (int i=0; i<nl.getLength(); i++) {
      Node n = nl.item(i);
      if ((n.getNodeType() == Node.ELEMENT_NODE) && (n.getNodeName().compareTo("architecture_ref")==0)) {
        return n.getFirstChild().getNodeValue();
      }
    }
    return null;
  }
}
