/**
 * BundleInputStream.java is a part of the SOFA project.
 * This file was created by pepan on 26.3.2003.
 */
package SOFA.SOFAnode.TR.Impl;

import SOFA.Util.Tools;

import java.io.*;
import java.util.Date;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

/**
 * Stores bundle entries (component name, implementation version and all necessary files) into
 * a zipped stream. Actually, it looks like a JAR file with the <code>MANIFEST.INF</code> file.
 * However, each entry in the manifest file describes rather one component (a few files)
 * than a single file.
 *
 * @author Petr Panuska
 */
class BundleInputStream extends JarInputStream {

  /**
   * A directory to store component files being unpacked from this stream (a zipped jar file).
   * This directory (including all files included) is deleted when the associated bundle
   * is going to be garbage collected.
   * @see BundleImpl#finalize
   */
  private File tempDir;

  /**
   * Last JAR entry read from this stream.
   */
  private JarEntry lastEntry;

  /**
   * The name of a JAR entry that has been read last.
   */
  private String lastName;

  /**
   * Has the end of a file been reached?
   */
  private boolean eof;

  /**
   * Prefix (characters before the <code>"/"</code>) of a JAR entry name
   * that has been read last from this stream.
   */
  private String lastPrefix;

  /**
   * Creates the input stream for storing bundle entries. It stores component files
   * (described by entries) into a temporarily created directory.
   * @param in the underlying stream.
   * @throws IOException when some IO operation fails.
   */
  public BundleInputStream (InputStream in) throws IOException {
    super(in, false);
    tempDir = new File(System.getProperty("java.io.tmpdir") + File.separator + "sofa" + Long.toString(new Date().getTime()) + ".bar");
    tempDir.mkdir();
    lastEntry = null;
    lastPrefix = null;
    lastName = null;
    eof = false;
  }

  /**
   * Returns the next bundle entry (a component) from this stream.
   * <p><b>NB</b>: Don't call the {@link #closeEntry} and {@link #read} methods!! This method already
   * reads data and closes entries.
   * @return the next bundle entry or <code>null</code> when the end of stream has been reached.
   * @throws IOException when some IO operation fails.
   */
  public BundleEntry getNextBundleEntry () throws IOException {
    if (eof)  // end of stream has been reached
      return null;
    if (lastEntry == null) { // this method has been called for the first time
      lastEntry = getNextJarEntry();
      if (lastEntry == null) {     // end of stream has been reached
        eof = true;
        return null;
      }
      lastName = lastEntry.getName();  // name of the JAR entry;
      lastPrefix = lastName.substring(0, lastName.indexOf('/'));  // prefix of the name
    }
    BundleEntry bundleEntry = new BundleEntry(lastPrefix);
    for (; ;) {
      String path = lastName.substring(0, lastName.lastIndexOf("/")); // the name without suffix
      path = path.replace('/', File.separatorChar); // change the slash to the right file separator
      File file = new File(tempDir, path);
      file.mkdirs();
      file = new File(tempDir, lastName);
      FileOutputStream outFile = new FileOutputStream(file);

      Tools.copyStream(this, outFile);

      super.closeEntry();
      outFile.close();
      file.setLastModified(lastEntry.getTime());     // keep the original time of creation
      String hashName = lastName.substring(lastPrefix.length());                    // rest of the name
      bundleEntry.addFile(file, hashName);
      lastEntry = getNextJarEntry();
      if (lastEntry == null) {     // end of stream has been reached
        eof = true;
        return bundleEntry;
      }
      lastName = lastEntry.getName();  // name of the JAR entry;
      String newPrefix = lastName.substring(0, lastName.indexOf('/'));  // prefix of the name
      if (!lastPrefix.equals(newPrefix)) {
        lastPrefix = newPrefix;
        return bundleEntry;
      }
    }
  }

  /**
   * Returns the root directory created temporarily for storing files of all components
   * being written to this input stream.
   * @return the temporary directory where component files stored in this stream are
   * written in.
   */
  File getTempRootDirectory () {
    return tempDir;
  }
}
