/**
 * ComponentFiles.java is a part of the SOFA project.
 * This file was created by pepan on 20.3.2003.
 */
package SOFA.SOFAnode.TR.Impl;

import java.io.File;
import java.io.Serializable;
import java.util.Hashtable;

/**
 * An internal representation of a component. It lists all files necessary for a component.
 * @author Petr Panuska
 */
class ComponentFiles implements Serializable {

  /**
   * List of files contained by this component. Each key is a {@link String} and
   * serves as a name of the file in a bundle (JAR file). The associated value is
   * of type {@link File} and directly denotes the file stored in the filesystem of the
   * SOFA node.
   */
  private Hashtable files;

  /**
   * Creates an instance of this class with no files marked to be a part of this component.
   */
  ComponentFiles () {
    files = new Hashtable();
  }

  /**
   * Adds a file or a hierarchy of files to this component. If the specified path denotes
   * a directory, it adds this directory as well as all sub-directories and files. Otherwise,
   * it adds the specified file only.
   * @param path string denoting a file or a directory being added.
   */
  void addFiles (String path, String name) {
    File file = new File(path);
    if (file.isDirectory()) {             // list files in a directory
      if (!path.endsWith(File.separator))
        path = path + File.separator;
      if (!name.endsWith(File.separator))
        name = name + File.separator;
      String[] subFiles = file.list();
      for (int i = 0; i < subFiles.length; i++) {
        String subFile = subFiles[i];
        addFiles(path + subFile, name + subFile);
      }
    } else
      files.put(name, file); // do not store directories
  }

  /**
   * Adds one file under a given unique name. It must not be a directory.
   * @param file the file being added.
   * @param name the unique name.
   */
  void addFile (File file, String name) {
    assert(file.isDirectory());
    files.put(name, file); // do not store directories
  }

  /**
   * Gets a list of files being part of this component.
   * @return the component files.
   * @see #files
   */
  Hashtable getFiles () {
    return files;
  }

  /**
   * Gets the assembly descriptor file of this component. It's the file called
   * <em>index.dc</em>.
   * @return the file describing the assembly of this component.
   */
  File getAssemblyDescriptorFile () {
    return (File) files.get(File.separator + "index.dc");
  }
}
